@@ -16,7 +16,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-abi_version=2.0.3
+abi_version=2.0.10
 
 # abi_plugins is the list of subdirectories that are plugins
 # The definition of a plugin is a directory containing a file called "plugin.m4".
@@ -563,11 +563,19 @@ EOF
 rm -f autogen.out
 rm -f autogen.err
 
-echo "Running libtoolize -f -c"
-(libtoolize -f -c >> $abi_topdir/autogen.out 2>> $abi_topdir/autogen.err) || {
-  echo "libtoolize failed! Unable to continue."
-  exit 1
-}
+if test "x`uname`" = "xDarwin"; then
+    echo "Running glibtoolize -f -c"
+    (glibtoolize -f -c >> $abi_topdir/autogen.out 2>> $abi_topdir/autogen.err) || {
+	echo "glibtoolize failed! Unable to continue."
+	exit 1
+    }
+else
+    echo "Running libtoolize -f -c"
+    (libtoolize -f -c >> $abi_topdir/autogen.out 2>> $abi_topdir/autogen.err) || {
+	echo "libtoolize failed! Unable to continue."
+	exit 1
+    }
+fi
 
 echo "Running aclocal -I ac-helpers $ACLOCAL_FLAGS"
 (aclocal -I ac-helpers $ACLOCAL_FLAGS >> $abi_topdir/autogen.out 2>> $abi_topdir/autogen.err) || {
@@ -575,16 +583,27 @@ echo "Running aclocal -I ac-helpers $ACLOCAL_FLAGS"
   exit 1
 }
 
-echo "Checking for PKG_CHECK_MODULES..."
+if test -f autom4te.cache/requests; then
+    echo "Checking for PKG_CHECK_MODULES in autom4te.cache/requests ..."
+    pkgcheckdef=`grep PKG_CHECK_MODULES autom4te.cache/requests`
+else
+    echo "Checking for PKG_CHECK_MODULES in aclocal.m4 ..."
+    pkgcheckdef=`grep PKG_CHECK_MODULES aclocal.m4 | grep AC_DEFUN`
+fi
 
-pkgcheckdef=`grep PKG_CHECK_MODULES aclocal.m4 | grep AC_DEFUN`
 if test "x$pkgcheckdef" = "x"; then
   echo "Running aclocal -I ac-helpers -I ac-helpers/pkg-config $ACLOCAL_FLAGS"
   (aclocal -I ac-helpers -I ac-helpers/pkg-config $ACLOCAL_FLAGS >> $abi_topdir/autogen.out 2>> $abi_topdir/autogen.err) || {
     echo "aclocal failed! Unable to continue."
     exit 1
   }
-  pkgcheckdef=`grep PKG_CHECK_MODULES aclocal.m4 | grep AC_DEFUN`
+  if test -f autom4te.cache/requests; then
+      echo "Checking for PKG_CHECK_MODULES in autom4te.cache/requests ..."
+      pkgcheckdef=`grep PKG_CHECK_MODULES autom4te.cache/requests`
+  else
+      echo "Checking for PKG_CHECK_MODULES in aclocal.m4 ..."
+      pkgcheckdef=`grep PKG_CHECK_MODULES aclocal.m4 | grep AC_DEFUN`
+  fi
   if test "x$pkgcheckdef" = "x"; then
     echo ""
     echo "error: PKG_CHECK_MODULES isn't defined"
